<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://example.com
 * @since      1.0.0
 *
 * @package    Plugin_Name
 * @subpackage Plugin_Name/admin
 */
/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Plugin_Name
 * @subpackage Plugin_Name/admin
 * @author     Your Name <email@example.com>
 */
class CARP_Admin {
	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;
	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;
	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		define('CARP_PLUGIN_DIR', plugin_dir_url(__FILE__));
	}
	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Plugin_Name_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Plugin_Name_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
	    wp_enqueue_style('wp-color-picker');
		wp_enqueue_style( 'carp_fonts_css', plugin_dir_url( __FILE__ ) . 'css/fonts.css ', array(), $this->version, 'all' );  
		wp_enqueue_style( 'carp_jquery_ui_css', plugin_dir_url( __FILE__ ) . 'css/jquery-ui.css', array(), $this->version, 'all' );
		wp_enqueue_style( 'carp_datatable_css', plugin_dir_url( __FILE__ ) . 'css/jquery.dataTables.css', array(), $this->version, 'all' );
		wp_enqueue_style( 'carp_datatable_css2', plugin_dir_url( __FILE__ ) . 'css/buttons.dataTables.css', array(), $this->version, 'all' );  
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/carp-admin.css', array('wp-color-picker'), $this->version, 'all' );
		wp_enqueue_style( 'carp_style_css', plugin_dir_url( __FILE__ ) . 'css/style.css', array(), $this->version, 'all' );
	}
	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Plugin_Name_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Plugin_Name_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		wp_enqueue_media();
		wp_enqueue_script( 'carp_jquery_ui_js', plugin_dir_url( __FILE__ ) . 'js/jquery-ui.min.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( 'carp_datatable_js', plugin_dir_url( __FILE__ ) . 'js/jquery.dataTables.min.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( 'carp_datatable_js1', plugin_dir_url( __FILE__ ) . 'js/dataTables.buttons.min.js', array( 'jquery' ), $this->version, false );  
		wp_enqueue_script( 'carp_datatable_js2', plugin_dir_url( __FILE__ ) . 'js/jszip.min.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( 'carp_datatable_js3', plugin_dir_url( __FILE__ ) . 'js/buttons.html5.min.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/carp-admin.js', array( 'jquery','wp-color-picker' ), $this->version, false );
		wp_localize_script( $this->plugin_name,'ajaxobject', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) );
		wp_localize_script($this->plugin_name,'image_urls', array( 'form_edit' => CARP_PLUGIN_DIR."images/",) );  
		wp_localize_script($this->plugin_name,'image_url', array( 'form__url' => CARP_PLUGIN_DIR,) );        
	}
	
	public function carp_admin_menu(){
		add_menu_page( 
			__( 'CF7 Autoresponder', 'cf7-autoresponder-addon' ),
			__( 'CF7 Autoresponder', 'cf7-autoresponder-addon' ),
			'manage_options',
			'carp_dashboard',
			array( $this, 'settings' ),
			'dashicons-format-aside', 
			999
		); 
		add_submenu_page(
	        'carp_dashboard',
	        'Dashboard',
	        'Dashboard',
	        'manage_options',
	        'carp_dashboard'
	    );
		add_submenu_page(
	        'carp_dashboard',
	        'Settings',
	        'Settings',
	        'manage_options',
	        'carp_settings',
	        array( $this, 'settings' )
	    );
		
		$setting_array = get_option('carp_subscriber_settings');
		$status=$this->check_status(get_option('carp_subscriber_status'));
		if( $status =="connect" ){
			if(post_type_exists( 'wpcf7_contact_form' )){
				add_submenu_page(
					'carp_dashboard',
					'Data Log',
					'Data Log',
					'manage_options',
					'carp_data_log',
					array( $this, 'settings' )
				);
			}
			add_submenu_page(
				'carp_dashboard',
				'Custom Template',
				'Custom Template',
				'manage_options',
				'carp_custom_template',
				array( $this, 'settings' )
			);
		}
	}
	
	public function settings(){
		global $wpdb; 
		if(isset($_GET['page']) && $_GET['page'] == 'carp_dashboard'){
			require_once plugin_dir_path( __FILE__ ) . 'partials/dashboard.php';  
		}
		if(isset($_GET['page']) && $_GET['page'] == 'carp_settings'){
			$setting_array = get_option('carp_subscriber_settings'); 
			$all_status = get_option('carp_subscriber_status');
			$status=$this->check_status(get_option('carp_subscriber_status'));
			require_once CARP_PATH . 'includes/subscriber.php';
			require_once plugin_dir_path( __FILE__ ) . 'partials/settings.php';
		}
		if(isset($_GET['page']) && $_GET['page'] == 'carp_data_log'){
			$setting_array = get_option('carp_subscriber_settings');
			$status=$this->check_status(get_option('carp_subscriber_status'));
			require_once plugin_dir_path( __FILE__ ) . 'partials/data_log.php';
		}
		if(isset($_GET['page']) && $_GET['page'] == 'carp_custom_template'){
			$setting_array = get_option('carp_subscriber_settings');
			$status=$this->check_status(get_option('carp_subscriber_status'));
			$all_status = get_option('carp_subscriber_status');
			require_once CARP_PATH . 'includes/subscriber.php';
			require_once plugin_dir_path( __FILE__ ) . 'partials/templates.php';   
		}
	}
	
	public function check_status($arr =array()){
		foreach ($arr as $key => $value){
			if($value == "connect"){
				return "connect";
				break;
			}
		}
	}
	
	public function save_subscriber_setting(){
		// print_r($_POST['apikey']);
		// die;
        if($_POST['responder'] == "Aweber"){
			require_once CARP_PATH . '/subscribers/Aweber/aweber_api.php';
			require_once CARP_PATH . 'includes/subscriber.php';
			$data = get_option('carp_subscriber_settings');
			$status = get_option('carp_subscriber_status');
			if(empty($_POST['responder'])){
				echo json_encode(array( 'error' => 'Error occur may be something wrong.'));
				return;
			}
			$str = $_POST['apikey'];
			$descr = '';
			try{
				list($consumer_key, $consumer_secret, $access_key, $access_secret) = AWeberAPI::getDataFromAweberID($str);
			}
			catch (AWeberAPIException $exc){
				list($consumer_key, $consumer_secret, $access_key, $access_secret) = null;
				if(isset($exc->message))
				{
					$descr = $exc->message;
					$descr = preg_replace('/http.*$/i', '', $descr);	 # strip labs.aweber.com documentation url from error message
					$descr = preg_replace('/[\.\!:]+.*$/i', '', $descr); # strip anything following a . : or ! character
					$descr = '('.$descr.')';
				}
			}catch (AWeberOAuthDataMissing $exc){
				list($consumer_key, $consumer_secret, $access_key, $access_secret) = null;
			}catch (AWeberException $exc){
				list($consumer_key, $consumer_secret, $access_key, $access_secret) = null;
			}
			if (!$access_secret) {
				//$return['Error'] = 'Unable to connect to your AWeber Account ' .$descr;
				echo json_encode(array('error'=>'Unable to connect to your AWeber Accoun.'.$descr));
				$data[$_POST['responder']] = array(
					'consumer_key' => '',
					'consumer_secret' => '',
					'access_key' => '',
					'access_secret' => '',
				);
				update_option('carp_subscriber_settings',$data);
				$status['Aweber'] =  'disconnect';
				update_option('carp_subscriber_status',$status);
				update_option("carp_response","true_a");     
			}else{
				$data[$_POST['responder']] = array(
					'consumer_key' => $consumer_key,
					'consumer_secret' => $consumer_secret,
					'access_key' => $access_key,
					'access_secret' => $access_secret,
				);
				$status['Aweber'] = 'connect';
				update_option('carp_subscriber_settings',$data);
				update_option('carp_subscriber_status',$status);
				echo json_encode(array('success'=>true));  
				update_option("carp_response","true");  
			}
			die();
		}		
		if($_POST['responder'] == "Benchmark" || $_POST['responder'] == "Sendinblue" || $_POST['responder'] == "Sendlane" || $_POST['responder'] == "Sendiio" || $_POST['responder'] == "Verticalresponse" || $_POST['responder'] == "Mailchimp" || $_POST['responder'] == "GetResponse" || $_POST['responder'] == "CampaignMonitor" || $_POST['responder'] == "iContact"  || $_POST['responder'] == "ConstantContact" ||   $_POST['responder'] == "ImnicaMail"  ||   $_POST['responder'] == "ActiveCampaign"  ){    
			require_once CARP_PATH . 'includes/subscriber.php';
		
			$data = get_option('carp_subscriber_settings');
			$status = get_option('carp_subscriber_status');
			if(empty($_POST['responder'])){
				echo json_encode(array( 'error' => 'Error occur may be something wrong.'));
				return;
			}
			
			if(isset($_POST['conne']) && $_POST['conne']=="Disconnect"){
				$data[$_POST['responder']] = $_POST['apikey'];				
				update_option('carp_subscriber_settings', $data);
				$status[$_POST['responder']] =  'disconnect';    				
				update_option('carp_subscriber_status',$status);
				echo json_encode(array('error'=>true));  
				die(); 
			}			
			if($_POST['responder'] == "iContact"){
    			if(!empty($_POST['apikey']['app_id'])){
    				$objlist = new carp_subscriber();
    				$list = $objlist->switch_responder($_POST['apikey'], 'getList', $_POST['responder'],''); 
    				$data[$_POST['responder']] = $_POST['apikey'];
    				$status[$_POST['responder']] = 'connect';
    				update_option('carp_subscriber_settings',$data);
    				update_option('carp_subscriber_status',$status);
    				echo json_encode(array('success'=>true));  
    				update_option("carp_response","true");  			
    			}else{
    				echo json_encode(array('error'=>'An error occurred while getting your list.'));
    				$data[$_POST['responder']] = array('app_id' =>'');
    				update_option('carp_subscriber_settings',$data);
    				$status[$_POST['responder']] =  'disconnect';
    				update_option('carp_subscriber_status',$status);
    				update_option("carp_response","true_a");     
    			} 
			}else if( $_POST['responder'] == "Verticalresponse"){
				
				$api_key = $_POST['apikey']['api_key'];
				$secret = $_POST['apikey']['secret'];
				$redirect_url = $_POST['apikey']['redirect_url'];
				$code = $_POST['apikey']['code'];
			
				$curl = curl_init();

				curl_setopt_array($curl, array(
				CURLOPT_URL => 'https://vrapi.verticalresponse.com/api/v1/oauth/access_token',
				CURLOPT_RETURNTRANSFER => true,
				CURLOPT_ENCODING => '',
				CURLOPT_MAXREDIRS => 10,
				CURLOPT_TIMEOUT => 0,
				CURLOPT_FOLLOWLOCATION => true,
				CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
				CURLOPT_CUSTOMREQUEST => 'POST',
				CURLOPT_POSTFIELDS => array('client_id' => $api_key,'client_secret' =>  $secret,'redirect_uri' =>$redirect_url ,'code' => $code),
				CURLOPT_HTTPHEADER => array(
					'Cookie: _oceans_session_p=9c6da45d6fe2a2137337e83e0a491e5e'
				),
				));

				$response = curl_exec($curl);			
				curl_close($curl);
				$tkn = json_decode($response,true);
				if(isset($tkn['error']['code']) == 400){
					echo json_encode(array('error'=>$tkn['error']['message'].' Please click Authenticate first.'));
    				$data[$_POST['responder']] = array('app_id' =>'');
    				update_option('carp_subscriber_settings',$data);
				}else{									
					$tkn = json_decode($response,true);
					$_POST['apikey']['access_token'] = isset($tkn['access_token']) ? $tkn['access_token'] : '';					
					$objlist = new carp_subscriber();
					$list = $objlist->switch_responder($_POST['apikey'], 'getList', $_POST['responder'],''); 				
					$data[$_POST['responder']] = $_POST['apikey'];
					$status[$_POST['responder']] = 'connect';
					update_option('carp_subscriber_settings',$data);
					update_option('carp_subscriber_status',$status);
					echo json_encode(array('success'=>true));  
					update_option("carp_response","true");  
				}
			}else{	
				
			    if(!empty($_POST['apikey'])){
    				$objlist = new carp_subscriber();
    				$list = $objlist->switch_responder($_POST['apikey'], 'getList', $_POST['responder'],''); 	
									
    				$data[$_POST['responder']] = $_POST['apikey'];					
    				$status[$_POST['responder']] = 'connect';
    				update_option('carp_subscriber_settings',$data);
    				update_option('carp_subscriber_status',$status);
    				echo json_encode(array('success'=>true));  
    				update_option("carp_response","true");  			
    			}else{					
    				echo json_encode(array('error'=>'An error occurred while getting your list.'));
    				$data[$_POST['responder']] = array('app_id' =>'');
    				update_option('carp_subscriber_settings',$data);
    				$status[$_POST['responder']] =  'disconnect';
    				update_option('carp_subscriber_status',$status);
    				update_option("carp_response","true_a");   
					
    			} 
			}
			die(); 
		}
		
		if($_POST['responder'] == "E-goi" ){     
			
			require_once CARP_PATH . 'includes/subscriber.php';  
			$data = get_option('carp_subscriber_settings');
			$status = get_option('carp_subscriber_status');
			if(empty($_POST['responder'])){
				echo json_encode(array( 'error' => 'Error occur may be something wrong.'));
				return;
			}
			
			if(empty($_POST['apikey'])){
				update_option('cmp_subscriber_settings', $data);
				die(); 
			}
			$objlist = new carp_subscriber();
			$list = $objlist->switch_responder($_POST['apikey'], 'getList', $_POST['responder'],''); 
			if(isset($list['error']) && !empty($list['error'])){
				echo json_encode(array('error'=>'An error occurred while getting your list.'));
				$data[$_POST['responder']] = array('app_id' =>'');
				update_option('carp_subscriber_settings',$data);
				$status[$_POST['responder']] =  'disconnect';
				update_option('carp_subscriber_status',$status);
				update_option("carp_response","true_a");     
				
			}else{
				$data[$_POST['responder']] = $_POST['apikey'];
				$status[$_POST['responder']] = 'connect';
				update_option('carp_subscriber_settings',$data);
				update_option('carp_subscriber_status',$status);
				echo json_encode(array('success'=>true));  
				update_option("carp_response","true");    
			} 
			die(); 
		}
		
	}
	public function add_new_form_setting(){

		if(isset($_POST['carp_add_select_ar_form'])){
			require_once CARP_PATH . 'includes/subscriber.php';
			$setting_array = get_option('carp_subscriber_settings');
			$autores = $_POST['carp_add_select_ar_form'];
			$objlist = new carp_subscriber();  
			$lists = $objlist->switch_responder($setting_array[$autores], 'getList',$autores,'');
			//print_r($lists);
			echo '<option value="">Select List</option>';
			if(!empty($lists)){
				foreach($lists['list'] as $key => $value){
					echo '<option value="'.$key.'">'.$value.'</option>';
				}
			}
			die();
		}
		
		if(isset($_POST['carp_add_new_form'])){
			$form_id = $_POST['carp_add_new_form'];
			$list_id = $_POST['carp_list_id'];
			$autores = $_POST['carp_add_autoresponder'];
			$result= $this->add_new_formfn($autores ,$form_id , $list_id );
			echo $result;
			die();
		}
		
		if(isset($_POST['carp_add_cf7_list'])){
			global $wpdb;
			$method = $_POST['carp_edit_content'];
			$list_id = $_POST['carp_add_cf7_list'];
			$form_id = $_POST['carp_add_cf7_form'];
			$auto_res = $_POST['carp_edit_ar'];
			$field_selected = json_encode($_POST['carp_add_cf7_selected']);
			$field_disabled =json_encode($_POST['carp_add_cf7_disabled']);
			$autoresponder_list = json_encode($_POST['carp_add_cf7_aweber']);  
			$wpdb->get_results("SELECT * FROM ".$wpdb->prefix."carp_form_data WHERE list_id = '".$list_id."' AND form_id = '".$form_id."' "); 
			if($wpdb->num_rows == 0){
				$data =array(
					'autoresponder' => $auto_res,
					'list_id' => $list_id,
					'form_id' => $form_id,
					'autoresponder_list' => $autoresponder_list, 
					'field_selected' => $field_selected,
					'field_disabled' => $field_disabled,
				); 
				$result = $wpdb->insert($wpdb->prefix . 'carp_form_data' , $data);  
				
			}else if($method == "edit"){
				$data = array( 'autoresponder_list' => $autoresponder_list,
					'field_selected' => $field_selected,
					'field_disabled' => $field_disabled, 
				);
                $where = array('form_id' => $form_id , 'form_id' => $form_id,  ); 
                $result = $wpdb->update($wpdb->prefix."carp_form_data", $data, $where); 
			}
			if($result){
				echo "Save Succrssfufully";
			}else{
				echo "fail";
			} 
			die();
		}
		
		if(isset($_POST['carp_delete_form'])){
			global $wpdb;
			$form_id =  $_POST['carp_delete_form'];
			$list_id = $_POST['carp_delete_form_list_id'];
			$result = $wpdb->delete($wpdb->prefix."carp_form_data", array('form_id' => $form_id , 'list_id' => $list_id )); 
			if($result){
				echo "success";
			}else{
				echo "fail";
			}
			die();
		}
		
		if(isset($_POST['carp_edit_form'])){
			global $wpdb;
			$form_id =  $_POST['carp_edit_form'];
			$list_id = $_POST['carp_edit_form_list_id'];
			$autores = $_POST['carp_edit_autors'];
			$method = $_POST['carp_method'];
			$result_array =array();
			$result =$wpdb->get_results("SELECT * FROM ".$wpdb->prefix."carp_form_data WHERE list_id = '".$list_id."' AND form_id = '".$form_id."' "); 
			if($wpdb->num_rows > 0){
				$list_id = $result[0]->list_id;
				$form_id = $result[0]->form_id;
				$result_array['selected'] = json_decode($result[0]->field_selected);
				$result_array['disabled'] = json_decode($result[0]->field_disabled);
				$result_array['autoresponser'] = json_decode($result[0]->autoresponder_list); 
				$result= $this->add_new_formfn($autores , $form_id , $list_id, $method , $result_array);  
			    echo $result;
			}
			die();
		}
		
	}
	
	public function carp_broadcast_formfilter(){  
		$list = $_POST['listname'];
		$api_key =  get_option('carp_subscriber_settings')['Aweber']; 
		try{
			$aweber = new AWeberAPI($api_key['consumer_key'], $api_key['consumer_secret']);
			$account = $aweber->getAccount($api_key['access_key'], $api_key['access_secret']);
			$lists = $account->lists->find(array('name' =>$list));   
		}catch (AWeberException $e) {
		}
	
		if (!count($account->lists)){
				echo '<div class="cfea_promote_div"><p>Did not find list</p></div>';
			} else {
				echo '<table  id="carp_cf7_broadcast"><thead> 
						<tr>
							<th>Title</td>
							<th><span>Sent at</span></th>
							<th><span>Scheduled at</span></th>
							<th> <span>Opened</span></th>
							<th><span>Clicked</span></th>
							<th><span>Bounced</span></th>  
							<th><span>Complaints</span></th> 
						</tr></thead><tbody>'; 
						
					if($list == "select"){ 
						
					  foreach($account->lists as $offset => $list) {
					  foreach($list->campaigns as $campaign){
						  if ($campaign->type == 'broadcast_campaign') {   
						?>
							<tr>
								<td><?php echo $campaign->subject; ?></td> 
								<td><?php echo   $this->convert_time(date('F j, Y h:iA', strtotime($campaign->sent_at))); ?></td>
								<td><?php echo   $this->convert_time(date('F j, Y h:iA', strtotime($campaign->scheduled_at))); ?></td>
								<td> <?php echo ($campaign->total_opens != '')?$campaign->total_opens:'0'; ?></td>
								<td><?php echo  ($campaign->total_clicks != '')?$campaign->total_clicks:'0'; ?></td>
								<td><?php echo  ($campaign->total_undelivered != '')?$campaign->total_undelivered:'0'; ?></td>
								<td><?php echo  ($campaign->total_spam_complaints != '')?$campaign->total_spam_complaints:'0';?></td>
							</tr>
					<?php
						  }
					  }
					  }
			
		           }else{
					  $list_ids = $lists[0];
					  foreach($list_ids->campaigns as $campaign){
						  if ($campaign->type == 'broadcast_campaign') {
						?>
							<tr>
								<td><?php echo $campaign->subject; ?></td>
								<td><?php echo $this->convert_time(date('F j, Y h:iA', strtotime($campaign->sent_at))); ?></td>
								<td><?php echo $this->convert_time(date('F j, Y h:iA', strtotime($campaign->scheduled_at))); ?></td>
								<td> <?php echo ($campaign->total_opens != '')?$campaign->total_opens:'0'; ?></td>
								<td><?php echo  ($campaign->total_clicks != '')?$campaign->total_clicks:'0'; ?></td>
								<td><?php echo  ($campaign->total_undelivered != '')?$campaign->total_undelivered:'0'; ?></td>
								<td><?php echo  ($campaign->total_spam_complaints != '')?$campaign->total_spam_complaints:'0';?></td>
							</tr>
					<?php
						}
					}
				  }
			echo '</tbody></table>';
			echo '<script>var  oTable1;
		oTable1 = jQuery("#carp_cf7_broadcast").dataTable({    
			"scrollX": true 
	    });  </script>';
		  } 
		die();
	}
	
	public function save_templete_setting(){
		
			if(isset($_POST['carp_temp_form'])){
				global $wpdb;
				$auters =  $_POST['carp_temp_autoresponder'];
				$list_id = $_POST['carp_temp_form'];
				$template = $_POST['carp_temp_template'];
				$data_temp = json_encode($_POST['carp_temp_data']); 
				$shortcode = $_POST['carp_temp_shortcode'];
				$result = $wpdb->get_results("SELECT * FROM ".$wpdb->prefix."carp_template_data WHERE list_id = '".$list_id."' and template_id = '".$template."'  ");
				if($wpdb->num_rows>0){
					$data =array(
						'data' => $data_temp,
					); 
					$where = array('autoresponder' => $auters ,'list_id' => $list_id , 'template_id' => $template,  ); 
					$result1 = $wpdb->update($wpdb->prefix."carp_template_data", $data, $where);
				}else{
					$data =array(
						'autoresponder' => $auters,
						'list_id' => $list_id,
						'template_id' => $template,
						'data' => $data_temp,
					); 
					$result1 = $wpdb->insert($wpdb->prefix . 'carp_template_data' , $data);  
				}
				if($result1){
					echo "Save Successfully";
				}else{
					echo "Form not submit successfully"; 
				}
				die();	 
			}
		
	}
	
	public function save_templete_select_list(){
		global $wpdb;
		if(isset($_POST['carp_temp_list'])){  
			$list_id = $_POST['carp_temp_list'];
			$template = $_POST['carp_temp_temp'];
			$result = $wpdb->get_results("SELECT * FROM ".$wpdb->prefix."carp_template_data WHERE list_id = '".$list_id."' and template_id = '".$template."'  ");
			if($wpdb->num_rows>0){
				$data = $result[0]->data;
				echo($data);
			}else{
				echo "fail";
			} 
			die();
		} 
	}
	
	public function save_templete_reset_list(){
		global $wpdb;
		if(isset($_POST['carp_reset_list'])){  
			$list_id = $_POST['carp_reset_list'];
			$template = $_POST['carp_reset_temp'];
			$result = $wpdb->delete($wpdb->prefix."carp_template_data", array('list_id' => $list_id , 'template_id' => $template ));   
			if($result){
				echo "success"; 
			}else{
				echo "fail";
			}
			die();
		}
	}
	
	public function add_new_formfn($autoresponder, $form_id , $list_id , $method = '' , $result_array = array() ){
		if($autoresponder == "Aweber" ){
			require_once CARP_PATH . '/subscribers/Aweber/aweber_api.php';
			$api_key =  get_option('carp_subscriber_settings')['Aweber']; 
			try{
				$aweber = new AWeberAPI($api_key['consumer_key'], $api_key['consumer_secret']);
				$account = $aweber->getAccount($api_key['access_key'], $api_key['access_secret']);
				$custom_fields =  $aweber->loadFromUrl('https://api.aweber.com/1.0/accounts/'. $account->id .'/lists/'.$list_id.'/custom_fields');
			}catch (AWeberException $e) {
			}	
		}else if($autoresponder == "Mailchimp"){
			require_once CARP_PATH . '/subscribers/Mailchimp/MailChimp.php';  
			$api_key =  get_option('carp_subscriber_settings')['Mailchimp']['api_key']; 
			$mcapi_conn = new Mailchimp($api_key); 
			$custom_fields = $mcapi_conn->get("lists/$list_id/merge-fields"); 
		}else if($autoresponder == "CampaignMonitor"){           
			require_once CARP_PATH . '/subscribers/CampaignMonitor/csrest_lists.php';
			$api_key =  get_option('carp_subscriber_settings')['CampaignMonitor']['api_key'];  
			$wrap = new CS_REST_Lists($list_id,  array('api_key' => $api_key) );               
			$cm_res = $wrap->get_custom_fields();  
			$custom_fields = $cm_res->response;
		}else if($autoresponder == "iContact"){
			require_once CARP_PATH . '/subscribers/iContact/iContactApi.php';
			$api_key =  get_option('carp_subscriber_settings')['iContact'];   
		}else if($autoresponder == "E-goi"){
			require_once CARP_PATH . '/subscribers/e-goi-wrapper-php-0.3.1/Egoi/Factory.php';
			require_once CARP_PATH . '/subscribers/e-goi-wrapper-php-0.3.1/Zend/XmlRpc/Client.php';
			$client = new Zend_XmlRpc_Client('http://api.e-goi.com/v2/xmlrpc.php');
			$apikey =  get_option('carp_subscriber_settings')['E-goi']['api_key'];
			$arguments = array(
				"apikey" => $apikey,
			); 
			$results = $client->call('getLists', array($arguments));
			if( count($results) > 0 ) {
				foreach ((array)$results as $v){
					if($v['listnum']  == $list_id ){
						$custom_fields = $v['extra_fields']; 
					} 
				}
			}
		}
	
		if(!empty($form_id)){
			$div ="<div style='display:none;'>".do_shortcode("[contact-form-7 id='".$form_id ."' ]")."</div>";
			$div .="<div style='display:none;' id='carp_add_new_method' >".$method."</div>";
			$div .='<div class="carp_field_boxes carp_field_boxes_1">
							<label>Autoresponder Fields</label>
							<ul id="carp_add_aweber_ul">';
							$div .='<li> EMAIL<input type="hidden" name="carp_form_fieldselect[]" value="EMAIL"  class="carp_draggable_input"  ></li>';
							$div .='<li> NAME<input type="hidden" name="carp_form_fieldselect[]" value="name"  class="carp_draggable_input"  ></li>';
					if(!empty($custom_fields)){
						$i = 0;
						foreach( $custom_fields as $list ){ 
							
					      if($autoresponder == "Aweber" ){
							  $list_ids = $list->name;
							   if($list_ids != 'EMAIL'){ 
									$div .='<li> '.$list_ids.'<input type="hidden" name="carp_form_fieldselect[]" value="'.$list_ids.'"  class="carp_draggable_input"  ></li>'; 
							   } 
						  }else if($autoresponder == "Mailchimp"){
							foreach( $list as $l ){
								$list_ids = isset($l['tag']) ? $l['tag'] : '';	

							    if($list_ids != 'EMAIL'){ 
									$div .='<li> '.$list_ids.'<input type="hidden" name="carp_form_fieldselect[]" value="'.$list_ids.'"  class="carp_draggable_input"  ></li>'; 
							    }	
							}					 					
						  }else  if($autoresponder == "CampaignMonitor"){  
								$div .='<li> '.$list->FieldName.'<input type="hidden" name="carp_form_fieldselect[]" value="'.$list->Key.'"  class="carp_draggable_input"  ></li>';    
						  }else  if($autoresponder == "E-goi"){  
								$div .='<li> '.$list["ref"].'<input type="hidden" name="carp_form_fieldselect[]" value="'.$list['id'].'"  class="carp_draggable_input"  ></li>';    
						  }		
						  $i++;														 
						}
						
					} 
		
			if(($autoresponder == "GetResponse" || $autoresponder == "iContact" || $autoresponder == "ConstantContact" ||  $autoresponder == "Verticalresponse"  )&&  $method == "edit"){
				 foreach($result_array['autoresponser'] as $value){
					 if($value !="EMAIL"  &&  $value !="name"  ){
						$div .='<li>'.$value.'<input type="hidden" name="carp_form_fieldselect[]" value="'.$value.'"  class="carp_draggable_input"><a href="link/to/trash/script/when/we/have/js/off" title="Delete this image" class="ui-icon ui-icon-trash" id="add_field_cf7_delete" >Delete image</a></li>';
					 }
				 }
			}
			$div .= '</ul>';  
		
			if($autoresponder == "GetResponse" || $autoresponder == "iContact" || $autoresponder == "ConstantContact" || $autoresponder == "Verticalresponse"){ 
				$div .= '<input type="text" id="add_new_custum_fields" class="carp_add_new_input"  value="" style="width:100%;"  placeholder="Enter Field Name" >';    
				$div .="<span class='carp_setted_messlab1'>Please enter field same as Autoresponder Field</span>"; 
                $div .='<button id="add_custum_field"  class="carp_btn">Add</button>'	;		
			}
			$div .= '</div>';    
				   
			if($method == "edit"){
				$div .='<div class="carp_field_boxes carp_field_boxes_2">
					<label>CF7 Enable Fields</label>
					<ul id="carp_add_new_enable_ul" class="carp_draggable">';
				$div .='</ul>
					</div>';
				$div .='<div class="carp_field_boxes carp_field_boxes_3">
					<span>CF7 Disable  Fields</span>
					<ul id="carp_add_new_disable_ul" class="carp_draggable">';
				$div .='</ul>
				</div>';
				$div .="<script> 
					
					jQuery(document).ready(function($){
						
						var cffield_arr = ".json_encode($result_array['selected']).";
						var cffield_arr_dis = ".json_encode($result_array['disabled']).";
						var res_arr =  new Array();
						$('.wpcf7 form :input').each(function(index, elm){
							if($(elm).attr('type')!='hidden' && $(elm).attr('type') !='submit'){
			
								if(cffield_arr !=''){ 
									var form_field = $(elm).attr('name');  
									if(jQuery.inArray( form_field, cffield_arr ) !== -1    ){
										res_arr[jQuery.inArray( form_field, cffield_arr ) ] = form_field;
									}else if(jQuery.inArray( form_field, cffield_arr_dis ) !== -1 ){
										var lab = ' '+form_field+'<br>';
										var check_fields = $('<li>').html(lab).prepend($('<input/>').attr({ type: 'hidden', value:form_field , name:'carp_form_fieldselect[]'}).addClass('carp_draggable_input'));
										$('#carp_add_new_disable_ul').append(check_fields);
									}else{
										var lab = ' '+form_field+'<br>';
										var check_fields = $('<li>').html(lab).prepend($('<input/>').attr({ type: 'hidden', value:form_field , name:'carp_form_fieldselect[]'}).addClass('carp_draggable_input'));
										$('#carp_add_new_disable_ul').append(check_fields);
									}
								}else{
									var form_field = $(elm).attr('name');  
									var lab = ' '+form_field+'<br>';
									var check_fields = $('<li>').html(lab).prepend($('<input/>').attr({ type: 'hidden', value:form_field , name:'carp_form_fieldselect[]'}).addClass('carp_draggable_input'));
									$('#carp_add_new_enable_ul').append(check_fields);
								}
							}				
						});
						jQuery.each( res_arr, function( i, val ) {
							if(val != undefined){
								var lab = ' '+val+'<br>';
								var check_fields = $('<li>').html(lab).prepend($('<input/>').attr({ type: 'hidden', value:val , name:'carp_form_fieldselect[]'}).addClass('carp_draggable_input'));
								$('#carp_add_new_enable_ul').append(check_fields); 
							} 
						});
						
						jQuery( '#carp_add_new_enable_ul , #carp_add_new_disable_ul' ).sortable({
							connectWith: '.carp_draggable',
							items: 'li:not(.ui-state-disabled)',
						}).disableSelection();
					});
				</script>";
			}else{
				$div .='<div class="carp_field_boxes carp_field_boxes_2">
						<label>CF7 Enable Fields</label>
						<ul id="carp_add_new_enable_ul" class="carp_draggable">';	
				$div .='</ul>
					</div>';	   
			
				$div .='<div class="carp_field_boxes carp_field_boxes_3">
						<span>CF7 Disable  Fields</span>
						<ul id="carp_add_new_disable_ul" class="carp_draggable">
						</ul>
					</div>';
				
				$div .="<script> 
					jQuery(document).ready(function($){
						$('.wpcf7 form :input').each(function(index, elm){
							if($(elm).attr('type')!='hidden' && $(elm).attr('type') !='submit'){
								var form_field = $(elm).attr('name');  
								var lab = ' '+form_field+'<br>';
								var check_fields = $('<li>').html(lab).prepend($('<input/>').attr({ type: 'hidden', value:form_field , name:'carp_form_fieldselect[]'}).addClass('carp_draggable_input'));
								$('#carp_add_new_enable_ul').append(check_fields);
							}				
						});
						jQuery( '#carp_add_new_enable_ul , #carp_add_new_disable_ul' ).sortable({
							connectWith: '.carp_draggable',
							items: 'li:not(.ui-state-disabled)',
						}).disableSelection();	 
					}); 
				</script>";
			}
		}
		return $div;  
		 
	}
	
	public function convert_time( $time ) {
		$timestamp = strtotime( $time ); // Converting time to Unix timestamp
		$offset = get_option( 'gmt_offset' ) * 60 * 60; // Time offset in seconds
		$local_timestamp = $timestamp + $offset;
		$local_time = date_i18n( 'Y-m-d H:i:s', $local_timestamp );  
		return $local_time;  
	}
	 
}